@@ -229,6 +229,7 @@ class PrestoRequest(object):
         # mypy cannot follow module import
         self._http_session = self.http.Session()  # type: ignore
         self._http_session.headers.update(self.http_headers)
+        self._http_cookies = {}
         self._auth = auth
         if self._auth:
             if http_scheme == constants.HTTP:
@@ -304,9 +305,19 @@ class PrestoRequest(object):
             ),
             max_attempts=self._max_attempts,
         )
-        self._get = with_retry(self._http_session.get)
-        self._post = with_retry(self._http_session.post)
-        self._delete = with_retry(self._http_session.delete)
+        def get(*args, **kwargs):
+            return self._http_session.get(
+                *args, cookies=self._http_cookies, **kwargs)
+        def post(*args, **kwargs):
+            return self._http_session.post(
+                *args, cookies=self._http_cookies, **kwargs)
+        def delete(*args, **kwargs):
+            return self._http_session.delete(
+                *args, cookies=self._http_cookies, **kwargs)
+
+        self._get = with_retry(get)
+        self._post = with_retry(post)
+        self._delete = with_retry(delete)
 
     def get_url(self, path):
         # type: (Text) -> Text
@@ -373,14 +384,14 @@ class PrestoRequest(object):
             proxies=PROXIES,
         )
 
-    def _process_error(self, error):
+    def _process_error(self, error, query_id):
         error_type = error['errorType']
         if error_type == 'EXTERNAL':
-            raise exceptions.PrestoExternalError(error)
+            raise exceptions.PrestoExternalError(error, query_id)
         elif error_type == 'USER_ERROR':
-            return exceptions.PrestoUserError(error)
+            return exceptions.PrestoUserError(error, query_id)
 
-        return exceptions.PrestoQueryError(error)
+        return exceptions.PrestoQueryError(error, query_id)
 
     def raise_response_error(self, http_response):
         if http_response.status_code == 503:
@@ -402,7 +413,7 @@ class PrestoRequest(object):
         response = http_response.json()
         logger.debug('HTTP {}: {}'.format(http_response.status_code, response))
         if 'error' in response:
-            raise self._process_error(response['error'])
+            raise self._process_error(response['error'], response.get('id'))
 
         if constants.HEADER_CLEAR_SESSION in http_response.headers:
             for prop in get_header_values(
@@ -418,6 +429,9 @@ class PrestoRequest(object):
                 ):
                 self._client_session.properties[key] = value
 
+        if http_response.cookies:
+            self._http_cookies.update(http_response.cookies)
+
         self._next_uri = response.get('nextUri')
 
         return PrestoStatus(
@@ -505,7 +519,10 @@ class PrestoQuery(object):
         call fetch() until is_finished is true.
         """
         if self._cancelled:
-            raise exceptions.PrestoUserError("Query has been cancelled")
+            raise exceptions.PrestoUserError(
+                "Query has been cancelled",
+                self.query_id,
+            )
 
         response = self._request.post(self._sql)
         status = self._request.process(response)
